WEAKNESS_BEFORE_STOP_WORDS = r' ((has|contains) (an |a |)|(was|were) (found|discovered) to contain a )'
WEAKNESS_PATTERN = r'(?P<weakness>.*?)'
WEAKNESS_AFTER_STOP_WORDS = r'( ((V|v)ulnerabilit(y|ies)|issue[s]?)( (was|were) (discovered|found)|)|)'

COMPONENT_BEFORE_STOP_WORDS = r' (in (the |its |)|(starting |)at )'
COMPONENT_PATTERN = r'(?P<component>.+?)'
COMPONENT_AFTER_STOP_WORDS = r'(file|component|feature|function|method)'

DATE_VERSION_PATTERN = r'(before|through) \d{4}-\d{2}-\d{2}'
VERSION_PATTERN = r'\b(?:v\.|v|R)*(?:\d+(?:\.(?:\d+|x))+)(?:\-\d|a)*\b'
COMMIT_VERSION_PATTERN = r'\b[0-9a-f]{5,40}\b'

CONFIGURATION_BEFORE_STOP_WORDS = r' (in|of|on) (some |the |a |)'
CONFIGURATION_PATTERN = (r'(?P<configuration>[a-zA-Z0-9-\/\(\)]{2,}.+?(?:' + VERSION_PATTERN + r'|' + COMMIT_VERSION_PATTERN
                         + r'|' + DATE_VERSION_PATTERN + r')(?: and (?:before|prior|earlier|below)| all versions|))')

ATTACKER_BEFORE_STOP_WORDS = r' (could |which |may |might |)(allow|enabl[e]?)(s|ed|ing|) (a |an |)'
ATTACKER_PATTERN = r'(?P<attacker>.*?(attacker|user|server)[s]?)'

IMPACT_BEFORE_STOP_WORDS = r' (to (cause |lead |execute |run |crash |overwrite |corrupt |take |perform |obtain |achieve |remotely |potentially (enable |))(a |)|resulting in|leading to |causing a )'
IMPACT_PATTERN = r'(?P<impact>.+?)'

VECTOR_BEFORE_STOP_WORDS = r' (via|by) (an |a |vectors |)'
VECTOR_PATTERN = r'(?P<vector>.+?)'

ROOT_CAUSE_BEFORE_STOP_WORD = r' (because (of|a|the|)|when|due to) '
ROOT_CAUSE_PATTERN = r'(?P<root_cause>.+?)'


FUNCTION_NAME_PATTERN = r'[a-zA-Z_][a-zA-Z0-9_:\(\)]+'
EXTENSION_PATTERN = r"\.[a-zA-Z0-9]{1,4}"
DIR_FILE_NAME_PATTERN = r"[a-zA-Z0-9_-]+"
FILE_PATTERN = DIR_FILE_NAME_PATTERN + EXTENSION_PATTERN
RELATIVE_PATH_PATTERN = DIR_FILE_NAME_PATTERN + r"(?:/(?:" + DIR_FILE_NAME_PATTERN + r"))+" + EXTENSION_PATTERN


REPLACEMENT_PHRASES = {
    r'via the (' + FUNCTION_NAME_PATTERN + r') function in the component (.+?)\.$': r'in the \1 function at \2 component.',
    r'via the (' + FUNCTION_NAME_PATTERN + r') (function|component) (at|in) (.+?)\.$': r'in the \1 at \4.',
    r'via the (function|component) (' + FUNCTION_NAME_PATTERN + r') (at|in) (.+?)\.$': r'in the \2 \1 at \4.',
    r'via (' + FUNCTION_NAME_PATTERN + r') class in (.+?)\.$': r'in the \1 at \2.',
    r'via the (' + FUNCTION_NAME_PATTERN + r') function.$': r'in the \1 function.',
    r'via the function (' + FUNCTION_NAME_PATTERN + r').$': r'in the \1 function.',
    r'via the component (.+?)\.$': r'in the \1.',
    r'in function (' + FUNCTION_NAME_PATTERN + r') in (.+?) in': r'in \1 function at \2 in',
    r'at (.+?) in (' + FUNCTION_NAME_PATTERN + r') in ' + CONFIGURATION_PATTERN: r'in \2 at \1 in \3',
    r'out of bounds': r'out-of-bounds',
    r'leads to a': r'leading to',
    r'^There is a': r'A',
    r'(\d+), ([a-z]+)': r'\1 \2',
    r'^In ' + CONFIGURATION_PATTERN + r' there is a': r'\1 has a',
    r'^(?:In |)' + CONFIGURATION_PATTERN + r' ((?:a |).+?)(?: vulnerability|) exists in': r'\1 has \3 in',
    r'^(' + FUNCTION_NAME_PATTERN + r') (in) (' + RELATIVE_PATH_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 \2 \3 file \4 ',
    r'(' + FUNCTION_NAME_PATTERN + r') function in (' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 function at \2 file \3 ',
    r'^(' + FUNCTION_NAME_PATTERN + r') (in) (' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 \2 \3 file \4 ',
    r'^(' + RELATIVE_PATH_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS: r'\1 file \2 ',
    r'^(' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS: r'\1 file \2 '
}

NOISE_PATTERNS = [
    r'\s+\(.*?\)',
    r'\s+\[CWE-\d{1,4}\]',
    r' a different vulnerability than CVE-\d{4}-\d{4,7}',
    r' a related issue to CVE-\d{4}-\d{4,7}',
    r' and CVE-\d{4}-\d{4,7}',
    r' fixed in ' + VERSION_PATTERN,
    r'can be abused '
]



import json
import pytest

from pathlib import Path
from cve_driller.driller import CVEDriller

data_folder = Path(__file__).absolute().parent.parent / "data"

inputs_folder = data_folder / "inputs"
outputs_folder = data_folder / "outputs"
driller = CVEDriller()


def get_test_cases():
    test_cases = []

    for starting_detail_dir in inputs_folder.iterdir():
        for filename in starting_detail_dir.iterdir():
            output_path = Path(outputs_folder, filename.parent.name, filename.stem + ".json")

            if output_path.exists():
                test_cases.append((filename, output_path))

    return test_cases


@pytest.mark.parametrize("input_file, expected_output_file", get_test_cases())
def test_extraction(input_file, expected_output_file):
    with open(input_file, "r", encoding="utf-8") as f:
        input_text = f.read()

    with open(expected_output_file, "r", encoding="utf-8") as f:
        expected_details = json.load(f)

    starting_detail, extracted_data = driller(input_text)

    assert starting_detail.value == expected_output_file.parent.name, f"Starting detail mismatch in {input_file}"
    assert extracted_data == expected_details, f"Pattern mismatch in {input_file}"



import json
import pandas as pd

from pathlib import Path

data_path = Path(__file__).parent.parent / "data" / "outputs"


def load_data_as_dataframe() -> pd.DataFrame:
    samples = []

    for folder in data_path.iterdir():
        if folder.is_dir():
            for file in folder.iterdir():
                if file.suffix == ".json":
                    with open(file, "r") as f:
                        sample = json.load(f)
                        sample["cve_id"] = file.stem
                        samples.append(sample)

    return pd.DataFrame(samples)


df = load_data_as_dataframe()

#for i, row in df[["cve_id", "impact"]].sort_values(by="cve_id").iterrows():
#    print(row[["cve_id", "impact"]].values)

print(df["impact"].value_counts())



import re

from typing import Tuple, Dict
from .common.enums import DetailType
from .patterns import STARTING_DETAILS_MAPPING, WEAKNESS_PATTERNS, CONFIGURATION_PATTERNS, COMPONENT_PATTERNS
from .patterns.common import REPLACEMENT_PHRASES, NOISE_PATTERNS


class CVEDriller:
    def __init__(self, verbose: bool = False):
        self.verbose = verbose

    def log_info(self, message: str):
        if self.verbose:
            print(message)

    def extract_from_patterns(self, desc: str, patterns: dict):
        for name, pattern in patterns.items():
            match = re.compile(pattern).match(desc)

            if match:
                self.log_info(f"{name} - {pattern}")
                return match.groupdict()

        return None

    @staticmethod
    def normalize_phrases(desc: str) -> str:
        if desc[-1] != '.':
            desc += '.'

        desc = re.sub(r'\s+', ' ', desc)
        desc = re.sub(r"\"", "", desc)

        for old, new in REPLACEMENT_PHRASES.items():
            desc = re.sub(old, new, desc)

        return desc

    @staticmethod
    def remove_noise(desc: str) -> str:
        for pattern in NOISE_PATTERNS:
            desc = re.sub(pattern, "", desc)

        return desc

    def __call__(self, desc: str) -> Tuple[DetailType | None, Dict[str, str] | None]:
        desc = desc.strip()
        desc = self.normalize_phrases(desc)
        desc = self.remove_noise(desc)

        self.log_info(f"Description: {desc}")
        for detail_type, pattern in STARTING_DETAILS_MAPPING.items():
            if re.match(pattern, desc):
                self.log_info(f"Found starting pattern: {detail_type}")

                if detail_type == DetailType.CONFIGURATION:
                    return DetailType.CONFIGURATION, self.extract_from_patterns(desc, CONFIGURATION_PATTERNS)
                elif detail_type == DetailType.COMPONENT:
                    return DetailType.COMPONENT, self.extract_from_patterns(desc, COMPONENT_PATTERNS)
                elif detail_type == DetailType.WEAKNESS:
                    return DetailType.WEAKNESS, self.extract_from_patterns(desc, WEAKNESS_PATTERNS)

        return None, None



import pandas as pd

from tqdm import tqdm
from pathlib import Path
from cve_driller.driller import CVEDriller

MULTI_SENTENCE_PATTERN = r'\w+\.\s+[A-Z0-9]'

if __name__ == "__main__":
    driller = CVEDriller()
    path = Path("~/Downloads/cve_impact_types.csv").expanduser()
    path_labels = Path("~/projects/nvdutils/bert_impact_type_labels.csv").expanduser()
    df = pd.read_csv(path, engine='python')
    df_labels = pd.read_csv(path_labels, engine='python')
    print(df['impact'].value_counts())
    df = df[df['impact'].isnull()]
    print(len(df))
    df.drop(columns=['impact'], inplace=True)

    df_labeled = df.merge(df_labels, how='left', left_on='cve-id', right_on='cve-id')
    print(df_labeled["impact"].value_counts())
    not_label_df = df_labeled[df_labeled['impact'] == "Not"]
    # Assuming your DataFrame is named df

    # To filter OUT rows matching the pattern:
    # filtered_df = not_label_df[~not_label_df['description'].str.contains(MULTI_SENTENCE_PATTERN, regex=True, na=False)]
    # print(len(filtered_df))
    impact_count = 0
    match_count = 0
    extracted_impact = []

    for i, row in tqdm(not_label_df.iterrows(), total=len(not_label_df)):
        detail_type, details = driller(row['description'])

        if details:
            match_count += 1

            if 'impact' in details:
                impact_count += 1
                extracted_impact.append(details['impact'])

    print(match_count, impact_count)
    print(extracted_impact)



from ..common.enums import DetailType
from .weakness import WEAKNESS_STARTING_DETAIL_PATTERN_2, WEAKNESS_PATTERNS
from .configuration import CONFIG_STARTING_DETAIL_PATTERN, CONFIGURATION_PATTERNS
from .component import COMPONENT_STARTING_DETAIL_PATTERN, COMPONENT_PATTERNS


STARTING_DETAILS_MAPPING = {
    DetailType.COMPONENT: COMPONENT_STARTING_DETAIL_PATTERN,
    DetailType.CONFIGURATION: CONFIG_STARTING_DETAIL_PATTERN,
    DetailType.WEAKNESS: WEAKNESS_STARTING_DETAIL_PATTERN_2
}








# component configuration weakness vector impact
# DETAILS_5_2 = (COMPONENT_PATTERN + CONFIGURATION_BEFORE_STOP_WORDS +
#               CONFIGURATION_PATTERN + WEAKNESS_BEFORE_STOP_WORDS + WEAKNESS_PATTERN +
#               VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.')




from .common import (WEAKNESS_BEFORE_STOP_WORDS, WEAKNESS_PATTERN, CONFIGURATION_PATTERN, ATTACKER_PATTERN,
                     ATTACKER_BEFORE_STOP_WORDS, COMPONENT_PATTERN, COMPONENT_BEFORE_STOP_WORDS, IMPACT_PATTERN,
                     IMPACT_BEFORE_STOP_WORDS, VECTOR_PATTERN, VECTOR_BEFORE_STOP_WORDS, ROOT_CAUSE_PATTERN,
                     ROOT_CAUSE_BEFORE_STOP_WORD)

# configuration weakness component
CONFIG_STARTING_DETAIL_PATTERN = (r'(?:In |)' + CONFIGURATION_PATTERN + WEAKNESS_BEFORE_STOP_WORDS +
                                  WEAKNESS_PATTERN + COMPONENT_BEFORE_STOP_WORDS + COMPONENT_PATTERN)

# configuration weakness component attacker impact vector
DETAILS_6_1 = (CONFIG_STARTING_DETAIL_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.')

# configuration weakness component attacker impact
DETAILS_5_1 = (CONFIG_STARTING_DETAIL_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.')

# configuration weakness component root_cause
DETAILS_4_1 = CONFIG_STARTING_DETAIL_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.$'

# configuration weakness component impact
DETAILS_4_2 = CONFIG_STARTING_DETAIL_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.'

# configuration weakness component vector
DETAILS_4_3 = CONFIG_STARTING_DETAIL_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$'

# configuration weakness component root_cause
DETAILS_4_4 = CONFIG_STARTING_DETAIL_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.'

# configuration weakness component
DETAILS_3_1 = CONFIG_STARTING_DETAIL_PATTERN + r'\.$'


CONFIGURATION_PATTERNS = {
    '6_1': DETAILS_6_1,
    '5_1': DETAILS_5_1,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2,
    '4_3': DETAILS_4_3,
    '4_4': DETAILS_4_4,
    '3_1': DETAILS_3_1
}



from .common import (WEAKNESS_BEFORE_STOP_WORDS, WEAKNESS_PATTERN, CONFIGURATION_PATTERN, COMPONENT_PATTERN,
                     IMPACT_PATTERN, IMPACT_BEFORE_STOP_WORDS, VECTOR_PATTERN, VECTOR_BEFORE_STOP_WORDS,
                     CONFIGURATION_BEFORE_STOP_WORDS, COMPONENT_AFTER_STOP_WORDS, ATTACKER_BEFORE_STOP_WORDS,
                     ATTACKER_PATTERN, ROOT_CAUSE_BEFORE_STOP_WORD, ROOT_CAUSE_PATTERN)


# component - config - weakness
COMPONENT_STARTING_DETAIL_PATTERN = (r'(?:The |)' + COMPONENT_PATTERN + COMPONENT_AFTER_STOP_WORDS
                                     + CONFIGURATION_BEFORE_STOP_WORDS + CONFIGURATION_PATTERN
                                     + WEAKNESS_BEFORE_STOP_WORDS + WEAKNESS_PATTERN)

# component - config - weakness - attacker - impact - vector
DETAILS_6_1 = (COMPONENT_STARTING_DETAIL_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.')


# component - config - weakness - vector - impact
DETAILS_5_1 = (COMPONENT_STARTING_DETAIL_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.')

# component - config - weakness - vector - root_cause
DETAILS_5_2 = (COMPONENT_STARTING_DETAIL_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN +
               ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.')

# component - config - weakness - vector
DETAILS_4_1 = COMPONENT_STARTING_DETAIL_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.'

# component - config - weakness - root_cause
DETAILS_4_2 = COMPONENT_STARTING_DETAIL_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.'

# component - config - weakness
DETAILS_3_1 = COMPONENT_STARTING_DETAIL_PATTERN + r'\.$'


COMPONENT_PATTERNS = {
    '6_1': DETAILS_6_1,
    '5_1': DETAILS_5_1,
    '5_2': DETAILS_5_2,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2,
    '3_1': DETAILS_3_1
}



from .common import (WEAKNESS_PATTERN, WEAKNESS_AFTER_STOP_WORDS, CONFIGURATION_PATTERN, VECTOR_BEFORE_STOP_WORDS,
                     CONFIGURATION_BEFORE_STOP_WORDS, COMPONENT_PATTERN, COMPONENT_BEFORE_STOP_WORDS, ATTACKER_PATTERN,
                     ATTACKER_BEFORE_STOP_WORDS, IMPACT_PATTERN, IMPACT_BEFORE_STOP_WORDS, VECTOR_PATTERN,
                     ROOT_CAUSE_BEFORE_STOP_WORD, ROOT_CAUSE_PATTERN)

# weakness component configuration
WEAKNESS_STARTING_DETAIL_PATTERN_1 = (r'(An |A |)' + WEAKNESS_PATTERN + WEAKNESS_AFTER_STOP_WORDS +
                                      COMPONENT_BEFORE_STOP_WORDS + COMPONENT_PATTERN +
                                      CONFIGURATION_BEFORE_STOP_WORDS + CONFIGURATION_PATTERN)

# weakness configuration
WEAKNESS_STARTING_DETAIL_PATTERN_2 = (r'(An |A |)' + WEAKNESS_PATTERN + WEAKNESS_AFTER_STOP_WORDS +
                                      CONFIGURATION_BEFORE_STOP_WORDS + CONFIGURATION_PATTERN)

# weakness component configuration attacker impact vector
DETAILS_6_1 = (WEAKNESS_STARTING_DETAIL_PATTERN_1 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.')

# weakness configuration attacker impact vector component
DETAILS_6_2 = (WEAKNESS_STARTING_DETAIL_PATTERN_2 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN +
               COMPONENT_BEFORE_STOP_WORDS + COMPONENT_PATTERN + r'\.')

# weakness component configuration attacker impact
DETAILS_5_1 = (WEAKNESS_STARTING_DETAIL_PATTERN_1 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.')

# weakness configuration attacker impact vector
DETAILS_5_2 = (WEAKNESS_STARTING_DETAIL_PATTERN_2 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.')

# weakness configuration attacker impact component
DETAILS_5_3 = (WEAKNESS_STARTING_DETAIL_PATTERN_2 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + COMPONENT_BEFORE_STOP_WORDS + COMPONENT_PATTERN + r'\.$')

# weakness configuration attacker impact
DETAILS_4_1 = (WEAKNESS_STARTING_DETAIL_PATTERN_2 + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
               IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.')

# weakness component configuration root_cause
DETAILS_4_2 = WEAKNESS_STARTING_DETAIL_PATTERN_1 + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.'

# weakness component configuration impact
DETAILS_4_3 = WEAKNESS_STARTING_DETAIL_PATTERN_1 + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.'

# weakness component configuration
DETAILS_3_1 = WEAKNESS_STARTING_DETAIL_PATTERN_1 + r'\.$'

# weakness configuration vector
DETAILS_3_2 = WEAKNESS_STARTING_DETAIL_PATTERN_2 + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$'


WEAKNESS_PATTERNS = {
    '6_1': DETAILS_6_1,
    '6_2': DETAILS_6_2,
    '5_1': DETAILS_5_1,
    '5_2': DETAILS_5_2,
    '5_3': DETAILS_5_3,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2,
    '4_3': DETAILS_4_3,
    '3_1': DETAILS_3_1,
    '3_2': DETAILS_3_2
}



import json

from cve_driller.driller import CVEDriller


if __name__ == "__main__":
    driller = CVEDriller(verbose=True)
    cve_id = "CVE-2019-3951"

    input_path = f"data/inputs/{cve_id}.txt"
    description = open(input_path, 'r').read()

    detail_type, details = driller(description)

    print(details)

    if details:
        output_path = f"data/outputs/{detail_type.value}/{cve_id}.json"

        with open(output_path, 'w') as f:
            json.dump(details, f, indent=4)
